@@ -15,6 +15,14 @@ GNU General Public License for more details.
 #include <openbabel/babelconfig.h>
 
 #include <openbabel/obmolecformat.h>
+#include <openbabel/mol.h>
+#include <openbabel/atom.h>
+#include <openbabel/bond.h>
+#include <openbabel/obiter.h>
+#include <openbabel/elements.h>
+#include <openbabel/internalcoord.h>
+#include <openbabel/generic.h>
+#include <cstdlib>
 
 using namespace std;
 namespace OpenBabel
@@ -60,7 +68,7 @@ namespace OpenBabel
   {
 
     OBMol* pmol = pOb->CastAndClear<OBMol>();
-    if(pmol==NULL)
+    if (pmol == nullptr)
       return false;
 
     //Define some references so we can use the old parameter names
@@ -76,7 +84,7 @@ namespace OpenBabel
     vector<double> charges;
     bool hasPartialCharges = false;
     double energy;
-    OBVectorData *dipoleMoment = NULL;
+    OBVectorData *dipoleMoment = nullptr;
     bool readingVibrations = false;
     vector< vector<vector3> > displacements; // vibrational displacements
     vector<double> frequencies, intensities;
@@ -92,20 +100,26 @@ namespace OpenBabel
     while	(ifs.getline(buffer,BUFF_SIZE))
       {
         // Avoid "FORCE CONSTANT IN CARTESIAN COORDINATES" (PR#3417992)
-        if(strstr(buffer,"  CARTESIAN COORDINATES") != NULL)
+        if (strstr(buffer, "  CARTESIAN COORDINATES") != nullptr)
           {
             // mol.EndModify();
             mol.Clear();
-            mol.BeginModify();
-            ifs.getline(buffer,BUFF_SIZE);	// blank
-            ifs.getline(buffer,BUFF_SIZE);	// column headings
             ifs.getline(buffer,BUFF_SIZE);	// blank
+
+            // could either be columns or real data
             ifs.getline(buffer,BUFF_SIZE);
-            tokenize(vs,buffer);
+            tokenize(vs, buffer);
+            if (vs.size() != 5 || vs[0][0] != '1') { // first character should be atom 1
+              // those were column headings
+              ifs.getline(buffer,BUFF_SIZE);	// blank
+              ifs.getline(buffer,BUFF_SIZE);
+              tokenize(vs,buffer);
+            }
+            // now we're at real data
             while (vs.size() == 5)
               {
                 atom = mol.NewAtom();
-                atom->SetAtomicNum(etab.GetAtomicNum(vs[1].c_str()));
+                atom->SetAtomicNum(OBElements::GetAtomicNum(vs[1].c_str()));
                 x = atof((char*)vs[2].c_str());
                 y = atof((char*)vs[3].c_str());
                 z = atof((char*)vs[4].c_str());
@@ -116,7 +130,31 @@ namespace OpenBabel
                 tokenize(vs,buffer);
               }
           }
-        else if(strstr(buffer,"UNIT CELL TRANSLATION") != NULL)
+        // ANGSTROMS but not DEGREES (cartesians, not angles)
+        else if (strstr(buffer, "(ANGSTROMS)") != nullptr && strstr(buffer, "(DEGREES)") == nullptr)
+          { // newer versions don't print CARTESIAN for final geometry
+            mol.Clear();
+            ifs.getline(buffer,BUFF_SIZE);	// blank
+            ifs.getline(buffer,BUFF_SIZE);
+            tokenize(vs,buffer);
+            while (vs.size() == 8)
+              {
+                if (strcmp(vs[1].c_str(), "Tv") != 0)
+                  {
+                    atom = mol.NewAtom();
+                    atom->SetAtomicNum(OBElements::GetAtomicNum(vs[1].c_str()));
+                    x = atof((char*)vs[2].c_str());
+                    y = atof((char*)vs[4].c_str());
+                    z = atof((char*)vs[6].c_str());
+                    atom->SetVector(x,y,z);
+                  }
+
+                if (!ifs.getline(buffer,BUFF_SIZE))
+                  break;
+                tokenize(vs,buffer);
+              }
+          }
+        else if (strstr(buffer, "UNIT CELL TRANSLATION") != nullptr)
           {
             numTranslationVectors = 0; // ignore old translationVectors
             ifs.getline(buffer,BUFF_SIZE);	// blank
@@ -136,7 +174,7 @@ namespace OpenBabel
               }
           }
         // Optimized translation vectors:
-        else if (strstr(buffer, "FINAL  POINT  AND  DERIVATIVES") != NULL)
+        else if (strstr(buffer, "FINAL  POINT  AND  DERIVATIVES") != nullptr)
           {
             numTranslationVectors = 0; // Reset
             ifs.getline(buffer,BUFF_SIZE);	// blank
@@ -182,14 +220,68 @@ namespace OpenBabel
                 tokenize(vs,buffer);
               }
           }
-        else if(strstr(buffer,"DOUBLY OCCUPIED LEVELS") != NULL)
+        else if (strstr(buffer, "NC:NB:NA:I") != nullptr) // z-matrix
+          {
+            mol.Clear();
+            vector<OBInternalCoord*> vic;
+            vector<unsigned int> indices;
+            vic.push_back(nullptr);
+
+            while (ifs.getline(buffer,BUFF_SIZE)) {
+              tokenize(vs,buffer);
+              if (vs.size() == 0)
+                break;
+              else if (vs.size() < 11)
+                break;
+
+              atom = mol.NewAtom();
+
+              OBInternalCoord *coord = new OBInternalCoord;
+              coord->_dst = atof(vs[2].c_str());
+              coord->_ang = atof(vs[4].c_str());
+              coord->_tor = atof(vs[6].c_str());
+              vic.push_back(coord);
+
+              indices.push_back(atoi(vs[8].c_str()));
+              indices.push_back(atoi(vs[9].c_str()));
+              indices.push_back(atoi(vs[10].c_str()));
+
+              // symbol in column 1
+              atom->SetAtomicNum(OBElements::GetAtomicNum(vs[1].c_str()));
+            }
+            // read the z-matrix
+
+            // now fill in the atom ids into the internal coords
+            unsigned int idx = 0;
+            FOR_ATOMS_OF_MOL (a, mol) {
+              if ((indices[idx] > 0) && (indices[idx] <= mol.NumAtoms()))
+                vic[a->GetIdx()]->_a = mol.GetAtom(indices[idx]);
+              else
+                vic[a->GetIdx()]->_a = nullptr;
+
+              if ((indices[idx+1] > 0) && (indices[idx+1] <= mol.NumAtoms()))
+                vic[a->GetIdx()]->_b = mol.GetAtom(indices[idx+1]);
+              else
+                vic[a->GetIdx()]->_b = nullptr;
+
+              if ((indices[idx+2] > 0) && (indices[idx+2] <= mol.NumAtoms()))
+                vic[a->GetIdx()]->_c = mol.GetAtom(indices[idx+2]);
+              else
+                vic[a->GetIdx()]->_c = nullptr;
+
+              idx += 3;
+            }
+            InternalToCartesian(vic,mol);
+            // coordinates should be set
+          }
+        else if (strstr(buffer, "DOUBLY OCCUPIED LEVELS") != nullptr)
           {
             tokenize(vs, buffer);
             if (vs.size() < 9)
               continue;
             alphaHOMO = atoi(vs[8].c_str());
           }
-        else if(strstr(buffer,"EIGENVALUES") != NULL)
+        else if (strstr(buffer, "EIGENVALUES") != nullptr)
           {
             ifs.getline(buffer, BUFF_SIZE); // real data
             tokenize(vs, buffer);
@@ -202,12 +294,34 @@ namespace OpenBabel
               tokenize(vs, buffer);
             }
           }
-        else if(strstr(buffer,"FINAL HEAT") != NULL)
+        else if (strstr(buffer, "FINAL HEAT") != nullptr)
           {
             sscanf(buffer,"%*s%*s%*s%*s%*s%lf",&energy);
             mol.SetEnergy(energy);
           }
-        else if(strstr(buffer,"NET ATOMIC CHARGES") != NULL)
+        else if (strstr(buffer, "ELECTROSTATIC POTENTIAL CHARGES") != nullptr)
+          {
+            hasPartialCharges = true;
+            charges.clear(); // Mulliken Charges
+            ifs.getline(buffer,BUFF_SIZE);	// blank
+            ifs.getline(buffer,BUFF_SIZE);	// column headings
+            ifs.getline(buffer,BUFF_SIZE);
+            tokenize(vs,buffer);
+            if (vs.size() < 1) return false; // timvdm 18/06/2008
+            while (vs.size() > 0 && strstr(vs[0].c_str(), "DIPOLE") == nullptr)
+              {
+                if (vs.size() < 3) break;
+                atom = mol.GetAtom(atoi(vs[0].c_str()));
+                if (atom != nullptr)
+                  atom->SetPartialCharge(atof(vs[2].c_str()));
+                charges.push_back(atof(vs[2].c_str()));
+
+                if (!ifs.getline(buffer,BUFF_SIZE))
+                  break;
+                tokenize(vs,buffer);
+              }
+          }
+        else if (strstr(buffer, "NET ATOMIC CHARGES") != nullptr)
           {
             hasPartialCharges = true;
             charges.clear();
@@ -216,11 +330,12 @@ namespace OpenBabel
             ifs.getline(buffer,BUFF_SIZE);
             tokenize(vs,buffer);
             if (vs.size() < 1) return false; // timvdm 18/06/2008
-            while (vs.size() > 0 && strstr(vs[0].c_str(),"DIPOLE") == NULL)
+            while (vs.size() > 0 && strstr(vs[0].c_str(), "DIPOLE") == nullptr)
               {
                 if (vs.size() < 3) break;
                 atom = mol.GetAtom(atoi(vs[0].c_str()));
-                atom->SetPartialCharge(atof(vs[2].c_str()));
+                if (atom != nullptr)
+                  atom->SetPartialCharge(atof(vs[2].c_str()));
                 charges.push_back(atof(vs[2].c_str()));
 
                 if (!ifs.getline(buffer,BUFF_SIZE))
@@ -229,7 +344,7 @@ namespace OpenBabel
               }
             // Now we should be at DIPOLE line. If missing, break out of block
             // and continue parsing file.
-            if (vs.size() == 0 || strstr(vs[0].c_str(), "DIPOLE") != NULL)
+            if (vs.size() == 0 || strstr(vs[0].c_str(), "DIPOLE") != nullptr)
               continue;
             if (!ifs.getline(buffer,BUFF_SIZE))	// POINT CHARGE
               continue; // let the outer loop handle this
@@ -253,13 +368,13 @@ namespace OpenBabel
             if (!ifs.getline(buffer,BUFF_SIZE))
               break;
           }
-        else if(strstr(buffer,"MASS-WEIGHTED COORDINATE ANALYSIS") != NULL)
+        else if(strstr(buffer,"MASS-WEIGHTED COORDINATE ANALYSIS") != nullptr)
           { // the correct vibrations -- earlier bits aren't mass-weighted
             readingVibrations = true;
             if (!ifs.getline(buffer,BUFF_SIZE))
               break;
           }
-        else if (readingVibrations && strstr(buffer, "Root No.") != NULL)
+        else if (readingVibrations && strstr(buffer, "Root No.") != nullptr)
           {
             ifs.getline(buffer, BUFF_SIZE); // blank line
             ifs.getline(buffer, BUFF_SIZE); // symmetry labels (for OB-2.3)
@@ -315,7 +430,7 @@ namespace OpenBabel
               modeCount = vs.size();
             }
           }
-        else if (readingVibrations && strstr(buffer, "T-DIPOLE") != NULL)
+        else if (readingVibrations && strstr(buffer, "T-DIPOLE") != nullptr)
           {
             unsigned int currentIntensity = intensities.size();
             tokenize(vs, buffer);
@@ -366,14 +481,6 @@ namespace OpenBabel
     // Attach unit cell translation vectors if found
     if (numTranslationVectors == 3) {
       OBUnitCell* uc = new OBUnitCell;
-      // Translation vectors are actually the translated positions of the
-      // first listed atom, so adjust for this:
-      if (mol.NumAtoms() > 0) {
-        const vector3 &atom1Pos = mol.GetAtomById(0)->GetVector();
-        translationVectors[0] -= atom1Pos;
-        translationVectors[1] -= atom1Pos;
-        translationVectors[2] -= atom1Pos;
-      }
       uc->SetData(translationVectors[0], translationVectors[1], translationVectors[2]);
       uc->SetOrigin(fileformatInput);
       mol.SetData(uc);
@@ -403,9 +510,9 @@ namespace OpenBabel
       OBConversion::RegisterFormat("mop",this, "chemical/x-mopac-input");
       OBConversion::RegisterFormat("mpc",this, "chemical/x-mopac-input");
       // Command-line keywords
-      OBConversion::RegisterOptionParam("k", NULL, 1, OBConversion::OUTOPTIONS);
+      OBConversion::RegisterOptionParam("k", nullptr, 1, OBConversion::OUTOPTIONS);
       // Command-line keyword file
-      OBConversion::RegisterOptionParam("f", NULL, 1, OBConversion::OUTOPTIONS);
+      OBConversion::RegisterOptionParam("f", nullptr, 1, OBConversion::OUTOPTIONS);
     }
 
     virtual const char* Description() //required
@@ -530,7 +637,7 @@ namespace OpenBabel
   {
 
     OBMol* pmol = pOb->CastAndClear<OBMol>();
-    if(pmol==NULL)
+    if (pmol == nullptr)
       return false;
 
     //Define some references so we can use the old parameter names
@@ -627,7 +734,7 @@ namespace OpenBabel
             atom = mol.NewAtom();
             atom->SetVector(x,y,z); //set coordinates
             //set atomic number
-            atom->SetAtomicNum(etab.GetAtomicNum(elementSymbol.c_str()));
+            atom->SetAtomicNum(OBElements::GetAtomicNum(elementSymbol.c_str()));
           }
       }
 
@@ -656,7 +763,7 @@ namespace OpenBabel
   bool MOPACCARTFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   {
     OBMol* pmol = dynamic_cast<OBMol*>(pOb);
-    if(pmol==NULL)
+    if (pmol == nullptr)
       return false;
 
     //Define some references so we can use the old parameter names
@@ -668,13 +775,11 @@ namespace OpenBabel
 
     const char *keywords = pConv->IsOption("k",OBConversion::OUTOPTIONS);
     const char *keywordFile = pConv->IsOption("f",OBConversion::OUTOPTIONS);
-    bool writeUnitCell = (NULL != pConv->IsOption("u", OBConversion::OUTOPTIONS));
+    bool writeUnitCell = nullptr != pConv->IsOption("u", OBConversion::OUTOPTIONS);
     string defaultKeywords = "PUT KEYWORDS HERE";
 
     if(keywords)
-      {
-        defaultKeywords = keywords;
-      }
+      defaultKeywords = keywords;
 
     if (keywordFile)
       {
@@ -686,8 +791,14 @@ namespace OpenBabel
               ofs << keyBuffer << endl;
           }
       }
-    else
-      ofs << defaultKeywords << endl;
+    else {
+      ofs << defaultKeywords;
+      if (mol.GetTotalCharge() != 0)
+        ofs << " CHARGE=" << mol.GetTotalCharge();
+      
+      // should handle GetTotalSpinMultiplicity() too
+      ofs << endl;
+    }
 
     ofs << mol.GetTitle() << endl;
     ofs << endl; // comment
@@ -696,7 +807,7 @@ namespace OpenBabel
     FOR_ATOMS_OF_MOL(atom, mol)
       {
         snprintf(buffer,BUFF_SIZE,"%-3s%8.5f 1 %8.5f 1 %8.5f 1",
-                 etab.GetSymbol(atom->GetAtomicNum()),
+                 OBElements::GetSymbol(atom->GetAtomicNum()),
                  atom->GetX(),
                  atom->GetY(),
                  atom->GetZ());
@@ -705,7 +816,7 @@ namespace OpenBabel
 
     OBUnitCell *uc = (OBUnitCell*)mol.GetData(OBGenericDataType::UnitCell);
     if (uc && writeUnitCell) {
-      uc->FillUnitCell(&mol); // complete the unit cell with symmetry-derived atoms
+      //      uc->FillUnitCell(&mol); // complete the unit cell with symmetry-derived atoms
 
       vector<vector3> cellVectors = uc->GetCellVectors();
       for (vector<vector3>::iterator i = cellVectors.begin(); i != cellVectors.end(); ++i) {
@@ -729,9 +840,9 @@ namespace OpenBabel
     {
       OBConversion::RegisterFormat("mopin", this, "chemical/x-mopac-input");
       // Command-line keywords
-      OBConversion::RegisterOptionParam("k", NULL, 1, OBConversion::OUTOPTIONS);
+      OBConversion::RegisterOptionParam("k", nullptr, 1, OBConversion::OUTOPTIONS);
       // Command-line keyword file
-      OBConversion::RegisterOptionParam("f", NULL, 1, OBConversion::OUTOPTIONS);
+      OBConversion::RegisterOptionParam("f", nullptr, 1, OBConversion::OUTOPTIONS);
     }
 
     virtual const char* Description() //required
@@ -758,7 +869,7 @@ namespace OpenBabel
   bool MOPACINTFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   {
     OBMol* pmol = pOb->CastAndClear<OBMol>();
-    if(pmol==NULL)
+    if (pmol == nullptr)
       return false;
 
     istream &ifs = *pConv->GetInStream();
@@ -771,7 +882,7 @@ namespace OpenBabel
 
     vector<OBInternalCoord*> vic;
     vector<unsigned int> indices;
-    vic.push_back((OBInternalCoord*)NULL);
+    vic.push_back(nullptr);
 
     ifs.getline(buffer,BUFF_SIZE); // keywords
     ifs.getline(buffer,BUFF_SIZE); // filename
@@ -800,7 +911,7 @@ namespace OpenBabel
       indices.push_back(atoi(vs[8].c_str()));
       indices.push_back(atoi(vs[9].c_str()));
 
-      atom->SetAtomicNum(etab.GetAtomicNum(vs[0].c_str()));
+      atom->SetAtomicNum(OBElements::GetAtomicNum(vs[0].c_str()));
     }
 
     unsigned int idx = 0;
@@ -808,17 +919,17 @@ namespace OpenBabel
       if ((indices[idx] > 0) && (indices[idx] <= mol.NumAtoms()))
         vic[a->GetIdx()]->_a = mol.GetAtom(indices[idx]);
       else
-        vic[a->GetIdx()]->_a = NULL;
+        vic[a->GetIdx()]->_a = nullptr;
 
       if ((indices[idx+1] > 0) && (indices[idx+1] <= mol.NumAtoms()))
         vic[a->GetIdx()]->_b = mol.GetAtom(indices[idx+1]);
       else
-        vic[a->GetIdx()]->_b = NULL;
+        vic[a->GetIdx()]->_b = nullptr;
 
       if ((indices[idx+2] > 0) && (indices[idx+2] <= mol.NumAtoms()))
         vic[a->GetIdx()]->_c = mol.GetAtom(indices[idx+2]);
       else
-        vic[a->GetIdx()]->_c = NULL;
+        vic[a->GetIdx()]->_c = nullptr;
 
       idx += 3;
     }
@@ -850,7 +961,7 @@ namespace OpenBabel
   bool MOPACINTFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   {
     OBMol* pmol = dynamic_cast<OBMol*>(pOb);
-    if(pmol==NULL)
+    if (pmol == nullptr)
       return false;
 
     ostream &ofs = *pConv->GetOutStream();
@@ -860,7 +971,7 @@ namespace OpenBabel
     OBAtom *a,*b,*c;
 
     vector<OBInternalCoord*> vic;
-    vic.push_back((OBInternalCoord*)NULL);
+    vic.push_back(nullptr);
 
     for (unsigned int i = 0; i<mol.NumAtoms(); i++)
       vic.push_back(new OBInternalCoord);
@@ -901,7 +1012,7 @@ namespace OpenBabel
       w = vic[atom->GetIdx()]->_ang;
       t = vic[atom->GetIdx()]->_tor;
 
-      strncpy(type, etab.GetSymbol(atom->GetAtomicNum()), 16);
+      strncpy(type, OBElements::GetSymbol(atom->GetAtomicNum()), 16);
       type[15] = '\0';
 
       if (t < 0)
