@@ -30,14 +30,13 @@ GNU General Public License for more details.
 #include <openbabel/rotor.h>
 #include <openbabel/obconversion.h>
 #include <openbabel/locale.h>
-#include <assert.h>
-
+#include <openbabel/distgeom.h>
+#include <openbabel/elements.h>
 
 #include <openbabel/stereo/stereo.h>
 #include <openbabel/stereo/cistrans.h>
 #include <openbabel/stereo/tetrahedral.h>
 #include <openbabel/stereo/squareplanar.h>
-
 /* OBBuilder::GetNewBondVector():
  * - is based on OBAtom::GetNewBondVector()
  * - but: when extending a long chain all the bonds are trans
@@ -70,13 +69,37 @@ namespace OpenBabel
       //
       \endcode
   **/
-  std::vector<std::pair<OBSmartsPattern*, std::vector<vector3> > > OBBuilder::_fragments;
+  //std::map<std::string, double> OBBuilder::_torsion;
+  std::vector<std::string> OBBuilder::_rigid_fragments;
+  std::map<std::string, int> OBBuilder::_rigid_fragments_index;
+  std::map<std::string, std::vector<vector3> > OBBuilder::_rigid_fragments_cache;
+  std::vector<std::pair<OBSmartsPattern*, std::vector<vector3> > > OBBuilder::_ring_fragments;
+
+  void OBBuilder::AddRingFragment(OBSmartsPattern *sp, const std::vector<vector3> &coords)
+  {
+    bool hasAllZeroCoords = true;
+    for (std::size_t i = 0; i < coords.size(); ++i) {
+      if (fabs(coords[i].x()) > 10e-8 ||
+          fabs(coords[i].y()) > 10e-8 ||
+          fabs(coords[i].z()) > 10e-8) {
+        hasAllZeroCoords = false;
+        break;
+      }
+    }
+
+    if (hasAllZeroCoords) {
+      std::stringstream ss;
+      ss << "Ring fragment " << sp->GetSMARTS() << " in ring-fragments.txt has all zero coordinates. Ignoring fragment.";
+      obErrorLog.ThrowError(__FUNCTION__, ss.str(), obError);
+    } else
+      _ring_fragments.push_back(pair<OBSmartsPattern*, vector<vector3> > (sp, coords));
+  }
 
   void OBBuilder::LoadFragments()  {
     // open data/fragments.txt
     ifstream ifs;
-    if (OpenDatafile(ifs, "fragments.txt").length() == 0) {
-      obErrorLog.ThrowError(__FUNCTION__, "Cannot open fragments.txt", obError);
+    if (OpenDatafile(ifs, "rigid-fragments-index.txt").length() == 0) {
+      obErrorLog.ThrowError(__FUNCTION__, "Cannot open ring-fragments-index.txt", obError);
       return;
     }
 
@@ -84,9 +107,21 @@ namespace OpenBabel
     // TODO: Use OpenDatafile()
     obLocale.SetLocale();
 
+    std::string smiles;
+    int index;
+    while(ifs >> smiles >> index) {
+      _rigid_fragments.push_back(smiles);
+      _rigid_fragments_index[smiles] = index;
+    }
+
+    if (OpenDatafile(ifs, "ring-fragments.txt").length() == 0) {
+      obErrorLog.ThrowError(__FUNCTION__, "Cannot open ring-fragments.txt", obError);
+      return;
+    }
+
     char buffer[BUFF_SIZE];
     vector<string> vs;
-    OBSmartsPattern *sp = NULL;
+    OBSmartsPattern *sp = nullptr;
     vector<vector3> coords;
     while (ifs.getline(buffer, BUFF_SIZE)) {
       if (buffer[0] == '#') // skip comment line (at the top)
@@ -95,27 +130,79 @@ namespace OpenBabel
       tokenize(vs, buffer);
 
       if (vs.size() == 1) { // SMARTS pattern
-        if (sp != NULL)
-          _fragments.push_back(pair<OBSmartsPattern*, vector<vector3> > (sp, coords));
+        if (sp != nullptr)
+          AddRingFragment(sp, coords);
 
         coords.clear();
         sp = new OBSmartsPattern;
         if (!sp->Init(vs[0])) {
           delete sp;
-          sp = NULL;
+          sp = nullptr;
           obErrorLog.ThrowError(__FUNCTION__, " Could not parse SMARTS from contribution data file", obInfo);
         }
       } else if (vs.size() == 3) { // XYZ coordinates
         vector3 coord(atof(vs[0].c_str()), atof(vs[1].c_str()), atof(vs[2].c_str()));
         coords.push_back(coord);
       }
-
     }
-
-    _fragments.push_back(pair<OBSmartsPattern*, vector<vector3> > (sp, coords));
+    AddRingFragment(sp, coords);
 
     // return the locale to the original one
     obLocale.RestoreLocale();
+
+    /*if (OpenDatafile(ifs, "torsion.txt").length() == 0) {
+      obErrorLog.ThrowError(__FUNCTION__, "Cannot open torsion.txt", obError);
+      return;
+    }
+    double angle;
+    while(ifs >> smiles >> angle) {
+      _torsion[smiles] = angle;
+    }
+    */
+  }
+
+  std::vector<vector3> OBBuilder::GetFragmentCoord(std::string smiles) {
+    if (_rigid_fragments_cache.count(smiles) > 0) {
+      return _rigid_fragments_cache[smiles];
+    }
+
+    std::vector<vector3> coords;
+    if (_rigid_fragments_index.count(smiles) == 0) {
+      return coords;
+    }
+
+    ifstream ifs;
+    if (OpenDatafile(ifs, "rigid-fragments.txt").length() == 0) {
+      obErrorLog.ThrowError(__FUNCTION__, "Cannot open rigid-fragments.txt", obError);
+      return coords;
+    }
+
+    ifs.clear();
+    ifs.seekg(_rigid_fragments_index[smiles]);
+    char buffer[BUFF_SIZE];
+    vector<string> vs;
+    bool hasAllZeroCoords = true;
+    while (ifs.getline(buffer, BUFF_SIZE)) {
+      tokenize(vs, buffer);
+      if (vs.size() == 4) { // XYZ coordinates
+        vector3 coord(atof(vs[1].c_str()), atof(vs[2].c_str()), atof(vs[3].c_str()));
+        if (fabs(coord.x()) > 10e-8 ||
+            fabs(coord.y()) > 10e-8 ||
+            fabs(coord.z()) > 10e-8)
+          hasAllZeroCoords = false;
+        coords.push_back(coord);
+      } else if (vs.size() == 1) { // SMARTS pattern
+        break;
+      }
+    }
+
+    if (hasAllZeroCoords) {
+      std::stringstream ss;
+      ss << "Rigid fragment " << smiles << " in rigid-fragments.txt has all zero coordinates.";
+      obErrorLog.ThrowError(__FUNCTION__, ss.str(), obError);
+    }
+
+    return coords;
   }
 
   vector3 GetCorrectedBondVector(OBAtom *atom1, OBAtom *atom2, int bondOrder = 1)
@@ -123,19 +210,22 @@ namespace OpenBabel
     double bondLength = 0.0;
 
     // We create an estimate of the bond length based on the two atoms
-    bondLength += etab.CorrectedBondRad(atom1->GetAtomicNum(), atom1->GetHyb());
-    bondLength += etab.CorrectedBondRad(atom2->GetAtomicNum(), atom2->GetHyb());
+    // Scaling is performed by the bond order corrections below
+    //  .. so we will use the straight covalent radii
+    bondLength += OBElements::GetCovalentRad(atom1->GetAtomicNum());
+    bondLength += OBElements::GetCovalentRad(atom2->GetAtomicNum());
 
     if (bondLength < 1.0)
       bondLength = 1.0;
 
     // These are based on OBBond::GetEquibLength
+    // Numbers come from averaged values of Pyykko and Atsumi
     if (bondOrder == -1) // aromatic
-      bondLength *= 0.93;
+      bondLength *= 0.9475;   // 0.9475 = average of 1.0 and 0.8950
     else if (bondOrder == 2)
-      bondLength *= 0.91;
+      bondLength *= 0.8950;   // 0.8950
     else if (bondOrder == 3)
-      bondLength *= 0.87;
+      bondLength *= 0.8578;   // 0.8578
 
     return OBBuilder::GetNewBondVector(atom1, bondLength);
   }
@@ -155,7 +245,7 @@ namespace OpenBabel
     bond4 = VZero;
     bond5 = VZero;
 
-    if (atom == NULL)
+    if (atom == nullptr)
       return VZero;
 
     int dimension = ((OBMol*)atom->GetParent())->GetDimension();
@@ -168,7 +258,7 @@ namespace OpenBabel
       //
       //  a   --->   a--*
       //
-      if (atom->GetValence() == 0) {
+      if (atom->GetExplicitDegree() == 0) {
         newbond = atom->GetVector() + VX * length;
         return newbond;
       }
@@ -183,20 +273,20 @@ namespace OpenBabel
       // make sure we place the new atom trans to a-2 (if there is an a-2 atom)
       //
       //   (a-2)             (a-2)
-      //     \                 \
+      //     \                 \                                             //
       //    (a-1)==a   --->   (a-1)==a          angle(a-1, a, *) = 120
-      //                              \
+      //                              \                                      //
       //                               *
       // hyb * = 3
       // ^^^^^^^^^
       // make sure we place the new atom trans to a-2 (if there is an a-2 atom)
       //
       //   (a-2)             (a-2)
-      //     \                 \
+      //     \                 \                                             //
       //    (a-1)--a   --->   (a-1)--a          angle(a-1, a, *) = 109
-      //                              \
+      //                              \                                      //
       //                               *
-      if (atom->GetValence() == 1) {
+      if (atom->GetExplicitDegree() == 1) {
         bool isCarboxylateO = atom->IsCarboxylOxygen();
 
         FOR_NBORS_OF_ATOM (nbr, atom) {
@@ -206,10 +296,12 @@ namespace OpenBabel
             continue;
 
           FOR_NBORS_OF_ATOM (nbr2, &*nbr) {
-            if (&*nbr2 != atom)
+            if (&*nbr2 != atom) {
               bond2 = nbr->GetVector() - nbr2->GetVector();
-            if (isCarboxylateO && nbr2->IsOxygen())
-              break; // make sure that the hydrogen is trans to the C=O
+
+              if (isCarboxylateO && nbr2->GetAtomicNum() == OBElements::Oxygen)
+                break; // make sure that the hydrogen is trans to the C=O
+            }
           }
         }
 
@@ -225,11 +317,8 @@ namespace OpenBabel
           }
           // there is no a-2 atom
           v1 = cross(bond1, vrand); // so find a perpendicular, given the random vector (this doesn't matter here)
-          v2 = cross(bond1, v1);
-        } else {
-          v1 = cross(bond1, bond2);
-          v2 = cross(bond1, v1);
         }
+        v2 = cross(bond1, v1);
         v2 = v2.normalize();
 
         // check to see if atom is a square planar in disguise
@@ -261,11 +350,11 @@ namespace OpenBabel
       }
 
       //
-      //    \	      \
+      //    \	      \                                                     //
       //     X  --->   X--*
       //    /         /
       //
-      if (atom->GetValence() == 2) {
+      if (atom->GetExplicitDegree() == 2) {
         FOR_NBORS_OF_ATOM (nbr, atom) {
           if (bond1 == VZero)
             bond1 = atom->GetVector() - nbr->GetVector();
@@ -280,13 +369,13 @@ namespace OpenBabel
 
         if (atom->GetHyb() == 2)
           newbond = v1;
-        if (atom->GetHyb() == 3) {
+        else if (atom->GetHyb() == 3) {
           v2 = cross(bond1, bond2); // find the perpendicular
           v2.normalize();
           newbond = bond1 - v2 * tan(DEG_TO_RAD*(180.0 - 109.471));
           newbond = v2 + v1 * (sqrt(2.0) / 2.0); // used to be tan(70.53 degrees/2) which is sqrt(2.0) / 2.0
         }
-        if (atom->GetHyb() == 5 || atom->GetHyb() == 4) {
+        else if (atom->GetHyb() == 5 || atom->GetHyb() == 4) {
           /* add the first equatorial atom, orthogonally to bond1 (and bond2 = -bond1) */
           /* is atom order correct?  I don't think it matters, but I might have to ask a chemist
            * whether PClF4 would be more likely to have an equatorial or axial Cl-P bond */
@@ -301,12 +390,12 @@ namespace OpenBabel
           v1 = v1.normalize();
           newbond = v1;
         }
-        if (atom->GetHyb() == 6) {
+        else if (atom->GetHyb() == 6) {
           v2 = cross(bond1, bond2);
           newbond = v2;
         }
 
-        newbond = newbond.normalize();
+        newbond = newbond.normalize(); //if newbond was not set, it will become non-finite here
         newbond *= length;
         newbond += atom->GetVector();
         return newbond;
@@ -335,7 +424,7 @@ namespace OpenBabel
        //    /          /
        //
        */
-      if (atom->GetValence() == 3) {
+      if (atom->GetExplicitDegree() == 3) {
         if (atom->GetHyb() == 3) {
           FOR_NBORS_OF_ATOM (nbr, atom) {
             if (bond1 == VZero)
@@ -419,7 +508,7 @@ namespace OpenBabel
         }
       }
 
-      if (atom->GetValence() == 4) {
+      if (atom->GetExplicitDegree() == 4) {
         if (atom->GetHyb() == 6) {
           FOR_NBORS_OF_ATOM (nbr, atom) {
             if (bond1 == VZero)
@@ -469,7 +558,7 @@ namespace OpenBabel
 
       }
 
-      if (atom->GetValence() == 5) {
+      if (atom->GetExplicitDegree() == 5) {
         if (atom->GetHyb() == 6) {
           FOR_NBORS_OF_ATOM (nbr, atom) {
             if (bond1 == VZero)
@@ -506,7 +595,7 @@ namespace OpenBabel
       //
       //  a   --->   a---*
       //
-      if (atom->GetValence() == 0) {
+      if (atom->GetExplicitDegree() == 0) {
         newbond = atom->GetVector() + VX * length;
         // Check that the vector is still finite before returning
         if (!isfinite(newbond.x()) || !isfinite(newbond.y()))
@@ -537,7 +626,7 @@ namespace OpenBabel
       //    (a-1)--a   --->   (a-1)--a          angle(a-1, a, *) = 109            //
       //                              \                                           //
       //                               *                                          //
-      if (atom->GetValence() == 1) {
+      if (atom->GetExplicitDegree() == 1) {
         OBAtom *nbr = atom->BeginNbrAtom(i);
         if (!nbr)
           return VZero;
@@ -559,14 +648,14 @@ namespace OpenBabel
         newbond *= length;
         newbond += atom->GetVector();
         return newbond;
-      } // GetValence() == 1
+      } // GetExplicitDegree() == 1
 
       //                          //
       //    \         \           //
       //     X  --->   X--*       //
       //    /         /           //
       //                          //
-      if (atom->GetValence() == 2) {
+      if (atom->GetExplicitDegree() == 2) {
         for (OBAtom *nbr = atom->BeginNbrAtom(i); nbr; nbr = atom->NextNbrAtom(i)) {
           if (bond1 == VZero)
             bond1 = atom->GetVector() - nbr->GetVector();
@@ -587,11 +676,11 @@ namespace OpenBabel
       //   --X  --->  --X--*      //
       //    /          /          //
       //                          //
-      if (atom->GetValence() == 3) {
+      if (atom->GetExplicitDegree() == 3) {
         OBStereoFacade stereoFacade((OBMol*)atom->GetParent());
         if (stereoFacade.HasTetrahedralStereo(atom->GetId())) {
-          OBBond *hash = 0;
-          OBBond *wedge = 0;
+          OBBond *hash = nullptr;
+          OBBond *wedge = nullptr;
           vector<OBBond*> plane;
           for (OBAtom *nbr = atom->BeginNbrAtom(i); nbr; nbr = atom->NextNbrAtom(i)) {
             OBBond *bond = atom->GetBond(nbr);
@@ -668,9 +757,9 @@ namespace OpenBabel
     OBAtom *a = mol.GetAtom(idxA);
     OBAtom *b = mol.GetAtom(idxB);
 
-    if (a == NULL)
+    if (a == nullptr)
       return false;
-    if (b == NULL)
+    if (b == nullptr)
       return false;
 
     OBBitVec fragment = GetFragment(b);
@@ -789,6 +878,8 @@ namespace OpenBabel
     }
     //
     // translate fragment
+    //for(vector<OBMol>::iterator f=fragments.begin(); f!=fragments.end(); f++) {
+    //
     //
     for (unsigned int i = 1; i <= mol.NumAtoms(); ++i) {
       if (fragment.BitIsSet(i)) {
@@ -801,12 +892,12 @@ namespace OpenBabel
     //
     // Get a neighbor of a and of b for setting the dihedral later
     //
-    OBAtom* nbr_a = NULL;
+    OBAtom* nbr_a = nullptr;
     FOR_NBORS_OF_ATOM(nbr, a) {
       nbr_a = &*nbr;
       break;
     }
-    OBAtom* nbr_b = NULL;
+    OBAtom* nbr_b = nullptr;
     FOR_NBORS_OF_ATOM(nbr, b) {
       if (fragment.BitIsSet(nbr->GetIdx())) {
         nbr_b = &*nbr;
@@ -823,15 +914,29 @@ namespace OpenBabel
     a->AddBond(bond);
     b->AddBond(bond);
     //
-    // Set the dihedral between the two fragments (for the moment, only handle double bonds)
+    // Set the dihedral between the two fragments
     //
     // For example, if a double bond is coming off a ring, then the dihedral
     // should be 180, e.g. for I/C=C\1/NC1 (don't worry about whether cis or trans
     // at this point - this will be corrected later)
     //
-    if(bondOrder==2 && a->GetHyb()==2 && b->GetHyb()==2 && nbr_a && nbr_b)
+    if (bondOrder == 2 && a->GetHyb() == 2 && b->GetHyb() == 2 && nbr_a &&
+        nbr_b)
       mol.SetTorsion(nbr_a, a, b, nbr_b, 180 * DEG_TO_RAD);
 
+    // another special case is a single bond between two sp2 carbons - twist
+    // e.g. biphenyl
+    if (bondOrder == 1 && a->GetHyb() == 2 && b->GetHyb() == 2 && nbr_a &&
+        nbr_b)
+      mol.SetTorsion(nbr_a, a, b, nbr_b, 45.0 * DEG_TO_RAD);
+
+    // another special case is building dihedrals between two rings
+    //  twist it a little bit (e.g., Platinum 00J_2XIR_A)
+    if (bondOrder == 1 && ((nbr_a && nbr_a->IsInRing() && b->IsInRing()) ||
+                           (nbr_b && nbr_b->IsInRing() && a->IsInRing())))
+      mol.SetTorsion(nbr_a, a, b, nbr_b, 60.0 * DEG_TO_RAD);
+    // TODO - other inter-fragment dihedrals here
+
     return true;
   }
 
@@ -841,27 +946,6 @@ namespace OpenBabel
     return Connect(mol, idxA, idxB, newpos, bondOrder);
   }
 
-
-  /*
-     matrix3x3 mat;
-     vector3 moldir = newpos - posa;
-     vector3 fragdir = GetNewBondVector(b); // b is at origin
-     moldir.normalize();
-     fragdir.normalize();
-     double angle = acos(dot(moldir, fragdir));
-     vector3 axis = cross(moldir, fragdir);
-     axis.normalize();
-
-     mat.RotAboutAxisByAngle(axis, angle);
-     for (unsigned int i = 1; i <= _workMol.NumAtoms(); ++i) {
-     if (fragment.BitIsSet(i)) {
-     vector3 tmpvec = _workMol.GetAtom(i)->GetVector();
-     tmpvec *= mat; //apply the rotation
-     _workMol.GetAtom(i)->SetVector(tmpvec);
-     }
-     }
-  */
-
   // Variation of OBBuilder::Swap that allows swapping with a vector3 rather than
   // an explicit bond. This is useful for correcting stereochemistry at Tet Centers
   // where it is sometimes necessary to swap an existing bond with the location
@@ -873,13 +957,13 @@ namespace OpenBabel
     OBAtom *c = mol.GetAtom(idxC);
 
     // make sure the atoms exist
-    if (a == NULL || b == NULL || c == NULL)
+    if (a == nullptr || b == nullptr || c == nullptr)
       return false;
 
     OBBond *bond1 = mol.GetBond(idxA, idxB);
 
     // make sure a-b is connected
-    if (bond1 == NULL)
+    if (bond1 == nullptr)
       return false;
 
     // make sure the bond are not in a ring
@@ -915,14 +999,14 @@ namespace OpenBabel
     OBAtom *d = mol.GetAtom(idxD);
 
     // make sure the atoms exist
-    if (a == NULL || b == NULL || c == NULL || d == NULL)
+    if (a == nullptr || b == nullptr || c == nullptr || d == nullptr)
       return false;
 
     OBBond *bond1 = mol.GetBond(idxA, idxB);
     OBBond *bond2 = mol.GetBond(idxC, idxD);
 
     // make sure a-b and c-d are connected
-    if (bond1 == NULL || bond2 == NULL)
+    if (bond1 == nullptr || bond2 == nullptr)
       return false;
 
     // make sure the bonds are not in a ring
@@ -986,20 +1070,24 @@ namespace OpenBabel
   //                                       b) Not the first atom: rotate, translate and connect the fragment
   // 3) The atom doesn't belong to a fragment: a) First atom: place at origin
   //                                           b) Not first atom: Find position and place atom
-  bool OBBuilder::Build(OBMol &mol)
+  bool OBBuilder::Build(OBMol &mol, bool stereoWarnings)
   {
-    //cerr << "OBBuilder::Build(OBMol &mol)" << endl;
     OBBitVec vdone; // Atoms that are done, need no further manipulation.
     OBBitVec vfrag; // Atoms that are part of a fragment found in the database.
                     // These atoms have coordinates, but the fragment still has
                     // to be rotated and translated.
     vector3 molvec, moldir;
-    vector<pair<OBSmartsPattern*, vector<vector3 > > >::iterator i;
     vector<vector<int> >::iterator j;
-    vector<int>::iterator k, k2, k3;
+    vector<int>::iterator k, k2;
     vector<vector3>::iterator l;
     vector<vector<int> > mlist; // match list for fragments
 
+    OBConversion conv;
+    conv.SetOutFormat("can"); // Canonical SMILES
+
+    // Trigger hybridisation perception now so it will be copied to workMol
+    mol.GetFirstAtom()->GetHyb();
+
     // copy the molecule to private data
     OBMol workMol = mol;
 
@@ -1007,106 +1095,151 @@ namespace OpenBabel
     if (workMol.GetDimension() == 2)
       workMol.SetDimension(0);
 
-    // Count the number of ring atoms.
-    unsigned int ratoms = 0;
-    FOR_ATOMS_OF_MOL(a, mol)
-      if (a->IsInRing()) {
-        ratoms++;
-        if (_keeprings) // Mark these as fragments
-          vfrag.SetBitOn(a->GetIdx());
-      }
 
-    if (_keeprings) {
-      // Delete all non-ring bonds
-      std::vector<OBBond*> for_deletion;
-      FOR_BONDS_OF_MOL(b, workMol)
-        if (!b->IsInRing())
-          for_deletion.push_back(&(*b));
-      for(std::vector<OBBond*>::iterator it=for_deletion.begin(); it!=for_deletion.end(); ++it) {
-        workMol.DeleteBond(*it);
-      }
-    }
-    else
-      // Delete all bonds in the working molecule
-      // (we will add them back at the end)
-      while (workMol.NumBonds())
-        workMol.DeleteBond(workMol.GetBond(0));
+    // Delete all bonds in the working molecule
+    // (we will add them back at the end)
+    while (workMol.NumBonds())
+      workMol.DeleteBond(workMol.GetBond(0));
 
+    // Deleting the bonds unsets HybridizationPerceived. To prevent our
+    // perceived values being reperceived (incorrectly), we must set
+    // this flag again.
     workMol.SetHybridizationPerceived();
 
-    if (ratoms && !_keeprings) {
-      //datafile is read only on first use of Build()
-      if(_fragments.empty())
-        LoadFragments();
 
-      // Skip all fragments that are too big to match
-      // Note: It would be better to compare to the size of the largest
-      //       isolated ring system instead of comparing to ratoms
-      for (i = _fragments.begin();i != _fragments.end() && i->first->NumAtoms() > ratoms;++i);
+    // I think just deleting rotable bond and separate is enough,
+    // but it did not work.
 
-      // Loop through  the remaining fragments and assign the coordinates from
-      // the first (most complex) fragment.
-      // Stop if there are no unassigned ring atoms (ratoms).
-      for (;i != _fragments.end() && ratoms;++i) {
+    // Get fragments using CopySubstructure
+    // Copy all atoms
+    OBBitVec atomsToCopy;
+    FOR_ATOMS_OF_MOL(atom, mol) {
+      atomsToCopy.SetBitOn(atom->GetIdx());
+    }
 
-        if (i->first != NULL && i->first->Match(mol)) {
-          mlist = i->first->GetUMapList();
+    // Exclude rotatable bonds
+    OBBitVec bondsToExclude;
+    FOR_BONDS_OF_MOL(bond, mol) {
+      if (bond->IsRotor()) {
+        bondsToExclude.SetBitOn(bond->GetIdx());
+      }
+    }
 
-          for (j = mlist.begin();j != mlist.end();++j) { // for all matches
+    // Generate fragments by copy
+    OBMol mol_copy;
+    mol.CopySubstructure(mol_copy, &atomsToCopy, &bondsToExclude);
 
+    // Separate each disconnected fragments as different molecules
+    vector<OBMol> fragments = mol_copy.Separate();
+
+    // datafile is read only on first use of Build()
+    if(_rigid_fragments.empty())
+      LoadFragments();
+
+
+    for(vector<OBMol>::iterator f = fragments.begin(); f != fragments.end(); ++f) {
+      std::string fragment_smiles = conv.WriteString(&*f, true);
+      bool isMatchRigid = false;
+      // if rigid fragment is in database
+      if (_rigid_fragments_index.count(fragment_smiles) > 0) {
+        OBSmartsPattern sp;
+        if (!sp.Init(fragment_smiles)) {
+          obErrorLog.ThrowError(__FUNCTION__, " Could not parse SMARTS from fragment", obInfo);
+        } else if (sp.Match(mol)) { // for all matches
+          isMatchRigid = true;
+          mlist = sp.GetUMapList();
+          for (j = mlist.begin(); j != mlist.end(); ++j) {
             // Have any atoms of this match already been added?
-            int alreadydone = 0;
-            int match_idx;
-            for (k = j->begin(); k != j->end(); ++k) // for all atoms of the fragment
+            bool alreadydone = false;
+            for (k = j->begin(); k != j->end(); ++k)
               if (vfrag.BitIsSet(*k)) {
-                alreadydone += 1;
-                if (alreadydone > 1) break;
-                match_idx = *k;
+                alreadydone = true;
+                break;
               }
-            bool spiro = false;
-            if (alreadydone > 1) // At least two atoms of the found match have already been added
-              continue;
-            else if (alreadydone == 1) {
-              spiro = IsSpiroAtom(mol.GetAtom(match_idx)->GetId(), mol);
-              if (!spiro) continue;
-            }
+            if (alreadydone) continue;
 
-            ratoms += alreadydone - static_cast<int> (j->size()); // Update the number of available ring atoms
             for (k = j->begin(); k != j->end(); ++k)
               vfrag.SetBitOn(*k); // Set vfrag for all atoms of fragment
 
-            if (spiro) {
-              vector<int> match_indices(1); // In future, it could be of longer length
-              match_indices[0] = match_idx;
-              ConnectFrags(mol, workMol, *j, i->second, match_indices);
+            int counter;
+            std::vector<vector3> coords = GetFragmentCoord(fragment_smiles);
+            for (k = j->begin(), counter=0; k != j->end(); ++k, ++counter) { // for all atoms of the fragment
+              // set coordinates for atoms
+              OBAtom *atom = workMol.GetAtom(*k);
+              atom->SetVector(coords[counter]);
             }
-            else {
+
+            // add the bonds for the fragment
+            for (k = j->begin(); k != j->end(); ++k) {
+              OBAtom *atom1 = mol.GetAtom(*k);
+              for (k2 = j->begin(); k2 != j->end(); ++k2) {
+                OBAtom *atom2 = mol.GetAtom(*k2);
+                OBBond *bond = atom1->GetBond(atom2);
+                if (bond != nullptr) {
+                  workMol.AddBond(*bond);
+                }
+              }
+            }
+          }
+        }
+      }
+      if(!isMatchRigid) {    // if rigid fragment is not in database
+        // Count the number of ring atoms.
+        unsigned int ratoms = 0;
+        FOR_ATOMS_OF_MOL(a, mol) {
+          if (a->IsInRing()) {
+            ratoms++;
+          }
+        }
+        if (ratoms < 3) continue; // Smallest ring fragment has 3 atoms
+
+        vector<pair<OBSmartsPattern*, vector<vector3 > > >::iterator i;
+        // Skip all fragments that are too big to match
+        // Note: It would be faster to compare to the size of the largest
+        //       isolated ring system instead of comparing to ratoms
+        for (i = _ring_fragments.begin(); i != _ring_fragments.end() && i->first->NumAtoms() > ratoms; ++i);
+
+        // Loop through the remaining fragments and assign the coordinates from
+        // the first (most complex) fragment.
+        // Stop if there are no unassigned ring atoms (ratoms).
+        for (; i != _ring_fragments.end() && ratoms; ++i) {
+          if (i->first != nullptr && i->first->Match(*f)) { // if match to fragment
+            i->first->Match(mol);                        // match over mol
+            mlist = i->first->GetUMapList();
+            for (j = mlist.begin();j != mlist.end();++j) { // for all matches
+              // Have any atoms of this match already been added?
+              bool alreadydone = false;
+              for (k = j->begin(); k != j->end(); ++k) { // for all atoms of the fragment
+                if (vfrag.BitIsSet(*k)) {
+                  alreadydone = true;
+                  break;
+                }
+              }
+              if (alreadydone) continue;
+              for (k = j->begin(); k != j->end(); ++k)
+                vfrag.SetBitOn(*k); // Set vfrag for all atoms of fragment
+
               int counter;
               for (k = j->begin(), counter=0; k != j->end(); ++k, ++counter) { // for all atoms of the fragment
                 // set coordinates for atoms
                 OBAtom *atom = workMol.GetAtom(*k);
                 atom->SetVector(i->second[counter]);
               }
-            }
-
-            // add the bonds for the fragment
-            int index2;
-            for (k = j->begin(); k != j->end(); ++k) {
-              OBAtom *atom1 = mol.GetAtom(*k);
-
-              for (k2 = j->begin(); k2 != j->end(); ++k2) {
-                index2 = *k2;
-                OBAtom *atom2 = mol.GetAtom(index2);
-                OBBond *bond = atom1->GetBond(atom2);
-
-                if (bond != NULL)
-                  workMol.AddBond(*bond);
+              // add the bonds for the fragment
+              for (k = j->begin(); k != j->end(); ++k) {
+                OBAtom *atom1 = mol.GetAtom(*k);
+                for (k2 = j->begin(); k2 != j->end(); ++k2) {
+                  OBAtom *atom2 = mol.GetAtom(*k2);
+                  OBBond *bond = atom1->GetBond(atom2);
+                  if (bond != nullptr)
+                    workMol.AddBond(*bond);
+                }
               }
             }
           }
         }
       }
-    } // if (ratoms)
+    } // for all fragments
 
     // iterate over all atoms to place them in 3D space
     FOR_DFS_OF_MOL (a, mol) {
@@ -1114,14 +1247,14 @@ namespace OpenBabel
         continue;
 
       // find an atom connected to the current atom that is already added
-      OBAtom *prev = NULL;
+      OBAtom *prev = nullptr;
       FOR_NBORS_OF_ATOM (nbr, &*a) {
         if (vdone.BitIsSet(nbr->GetIdx()))
           prev = &*nbr;
       }
 
       if (vfrag.BitIsSet(a->GetIdx())) { // Is this atom part of a fragment?
-        if (prev != NULL) { // if we have a previous atom, translate/rotate the fragment and connect it
+        if (prev != nullptr) { // if we have a previous atom, translate/rotate the fragment and connect it
           Connect(workMol, prev->GetIdx(), a->GetIdx(), mol.GetBond(prev, &*a)->GetBondOrder());
           // set the correct bond order
           int bondOrder = mol.GetBond(prev->GetIdx(), a->GetIdx())->GetBondOrder();
@@ -1139,8 +1272,8 @@ namespace OpenBabel
       //
 
       // get the position for the new atom, this is done with GetNewBondVector
-      if (prev != NULL) {
-        int bondType = a->GetBond(prev)->GetBO();
+      if (prev != nullptr) {
+        int bondType = a->GetBond(prev)->GetBondOrder();
         if (a->GetBond(prev)->IsAromatic())
           bondType = -1;
 
@@ -1165,7 +1298,7 @@ namespace OpenBabel
       atom->SetVector(molvec);
 
       // add bond between previous part and added atom
-      if (prev != NULL) {
+      if (prev != nullptr) {
         OBBond *bond = a->GetBond(prev); // from mol
         workMol.AddBond(*bond);
       }
@@ -1182,17 +1315,78 @@ namespace OpenBabel
     FOR_BONDS_OF_MOL(b, mol) {
       beginIdx = b->GetBeginAtomIdx();
       endIdx = b->GetEndAtomIdx();
-      workMol.AddBond(beginIdx, endIdx, b->GetBO(), b->GetFlags());
+      workMol.AddBond(beginIdx, endIdx, b->GetBondOrder(), b->GetFlags());
     }
 
+    /*
+    FOR_BONDS_OF_MOL(bond, mol) {
+      if(bond->IsRotor()) {
+        OBBitVec atomsToCopy;
+        OBAtom *atom = bond->GetBeginAtom();
+        FOR_NBORS_OF_ATOM(a, &*atom) {
+          atomsToCopy.SetBitOn(a->GetIdx());
+        }
+        atom = bond->GetEndAtom();
+        FOR_NBORS_OF_ATOM(a, &*atom) {
+          atomsToCopy.SetBitOn(a->GetIdx());
+        }
+        OBMol mol_copy;
+        mol.CopySubstructure(mol_copy, &atomsToCopy);
+        string smiles = conv.WriteString(&mol_copy, true);
+
+        if(_torsion.count(smiles) > 0) {
+          OBAtom* b = bond->GetBeginAtom();
+          OBAtom* c = bond->GetEndAtom();
+          OBAtom* a = nullptr;
+          FOR_NBORS_OF_ATOM(t, &*b) {
+            a = &*t;
+            if(a != c)
+              break;
+          }
+          OBAtom* d = nullptr;
+          FOR_NBORS_OF_ATOM(t, &*c) {
+            d = &*t;
+            if(d != b)
+              break;
+          }
+          double angle = _torsion[smiles] * DEG_TO_RAD;
+          mol.SetTorsion(a, b, c, d, angle);
+        } else {
+          ; // Do something
+        }
+      }
+    }
+    */
+
+    // We may have to change these success check
     // correct the chirality
-    CorrectStereoBonds(workMol);
-    CorrectStereoAtoms(workMol);
-    workMol.SetChiralityPerceived();
+    bool success = CorrectStereoBonds(workMol);
+    // we only succeed if we corrected all stereochemistry
+    success = success && CorrectStereoAtoms(workMol, stereoWarnings);
+
+    /*
+    // if the stereo failed, we should use distance geometry instead
+    OBDistanceGeometry dg;
+    dg.Setup(workMol);
+    dg.GetGeometry(workMol); // ensured to have correct stereo
+    */
+
     mol = workMol;
+    mol.SetChiralityPerceived();
     mol.SetDimension(3);
 
-    return true;
+    bool isNanExist = false;
+    FOR_ATOMS_OF_MOL(a, mol) {
+      vector3 v = a->GetVector();
+      if(IsNan(v.x()) || IsNan(v.y()) || IsNan(v.z())) {
+          isNanExist = true;
+          break;
+       }
+    }
+    if(isNanExist)
+      obErrorLog.ThrowError(__FUNCTION__, "There exists NaN in calculated coordinates.", obWarning);
+
+    return success;
   }
 
   void OBBuilder::ConnectFrags(OBMol &mol, OBMol &workMol, vector<int> match, vector<vector3> coords,
@@ -1223,7 +1417,7 @@ namespace OpenBabel
     //                                        \   /
     //  \        \   /    bisect  \             P    align   \                /
     //   P  and    P       --->    P--v1  and   |    --->     P--v1  and v2--P
-    //  /                         /             v2           /                \
+    //  /                         /             v2           /                \  //
 
     // Get v1 (from the existing fragment)
     vector3 bond1, bond2, bond3, bond4, v1;
@@ -1299,7 +1493,7 @@ namespace OpenBabel
     return;
   }
 
-  void OBBuilder::CorrectStereoBonds(OBMol &mol)
+  bool OBBuilder::CorrectStereoBonds(OBMol &mol)
   {
     // Get CisTransStereos and make a vector of corresponding OBStereoUnits
     std::vector<OBCisTransStereo*> cistrans, newcistrans;
@@ -1326,12 +1520,13 @@ namespace OpenBabel
     std::vector<OBCisTransStereo*>::iterator origct, newct;
     for (origct=cistrans.begin(), newct=newcistrans.begin(); origct!=cistrans.end(); ++origct, ++newct) {
       OBCisTransStereo::Config config = (*newct)->GetConfig(OBStereo::ShapeU);
+
       if ((*origct)->GetConfig(OBStereo::ShapeU) != config) { // Wrong cis/trans stereochemistry
 
         // refs[0]            refs[3]
         //        \          /
         //         begin==end
-        //        /          \
+        //        /          \                                                /
         // refs[1]            refs[2]
 
         a = mol.GetAtomById(config.refs[0]);
@@ -1343,34 +1538,36 @@ namespace OpenBabel
           d = mol.GetAtomById(config.refs[2]);
         angle = mol.GetTorsion(a, b, c, d); // In degrees
         newangle = angle * DEG_TO_RAD + M_PI; // flip the bond by 180 deg (PI radians)
+        // if it's a ring, break a ring bond before rotating
         mol.SetTorsion(a, b, c, d, newangle); // In radians
       }
     }
+
+    return true; // was all the ring bond stereochemistry corrected?
   }
 
   bool OBBuilder::IsSpiroAtom(unsigned long atomId, OBMol &mol)
   {
     OBMol workmol = mol; // Make a copy (this invalidates Ids, but not Idxs)
     OBAtom* watom = workmol.GetAtom(mol.GetAtomById(atomId)->GetIdx());
-    if (watom->GetHvyValence() != 4) // QUESTION: Do I need to restrict it further?
+    if (watom->GetHvyDegree() != 4) // QUESTION: Do I need to restrict it further?
       return false;
 
-    std::vector <OBBond*> bonds;
-    std::vector <OBBond*>::iterator bond_it;
-    FOR_BONDS_OF_ATOM(b, watom) {
-      if (!b->IsInRing())
+    int atomsInSameRing = 0;
+    int atomsInDiffRings = 0;
+    FOR_NBORS_OF_ATOM(n, watom) {
+      if (!n->IsInRing())
         return false;
+      if (mol.AreInSameRing(&*n, watom))
+        atomsInSameRing++;
       else
-        bonds.push_back(&(*b));
+        atomsInDiffRings++;
     }
-    // Removing the four bonds should partition the molecule into 3 fragments
-    // ASSUMPTION: Molecule is a contiguous fragment to begin with
-    for(bond_it=bonds.begin(); bond_it != bonds.end(); ++bond_it)
-      workmol.DeleteBond(*bond_it);
-    if (workmol.Separate().size() != 3)
-      return false;
 
-    return true;
+    if (atomsInSameRing == 2 && atomsInDiffRings == 2)
+      return true;
+
+    return false;
   }
 
   void OBBuilder::FlipSpiro(OBMol &mol, int idx)
@@ -1429,8 +1626,10 @@ namespace OpenBabel
         mol.GetAtom(i)->SetVector(workMol.GetAtom(i)->GetVector() + posP);
   }
 
-  void OBBuilder::CorrectStereoAtoms(OBMol &mol)
+  bool OBBuilder::CorrectStereoAtoms(OBMol &mol, bool warn)
   {
+    bool success = true; // for now
+
     // Get TetrahedralStereos and make a vector of corresponding OBStereoUnits
     std::vector<OBTetrahedralStereo*> tetra, newtetra;
     OBStereoUnitSet sgunits;
@@ -1485,16 +1684,18 @@ namespace OpenBabel
       bool inversion = FixRingStereo(ringstereo, mol, unfixed);
 
       // Output warning message if necessary
-      if (unfixed.size() > 0) {
+      if (unfixed.size() > 0 && warn) {
         stringstream errorMsg;
         errorMsg << "Could not correct " << unfixed.size() << " stereocenter(s) in this molecule (" << mol.GetTitle() << ")";
         errorMsg << std::endl << "  with Atom Ids as follows:";
         for (OBStereo::RefIter ref=unfixed.begin(); ref!=unfixed.end(); ++ref)
           errorMsg << " " << *ref;
         obErrorLog.ThrowError(__FUNCTION__, errorMsg.str(), obWarning);
+
+        success = false; // uncorrected bond
       }
 
-      // Reperceive non-ring TetrahedralStereos if an inversion occured
+      // Reperceive non-ring TetrahedralStereos if an inversion occurred
       if (inversion) {
         sgunits.clear();
         for (origth = nonringtetra.begin(); origth != nonringtetra.end(); ++origth)
@@ -1533,6 +1734,8 @@ namespace OpenBabel
         }
       }
     }
+
+    return success; // did we fix all atoms, including ring stereo?
   }
 
   bool OBBuilder::FixRingStereo(std::vector<std::pair<OBStereo::Ref, bool> > atomIds, OBMol &mol,
